import { world, system } from "@minecraft/server";


// This is what will reference when using registerEnchant
// You can use this to set and enchantment level and get if its on an item with the level attached
// For more information on how they work, look at an enchantment
export class Enchantment {
    constructor (id,name,maxLevel,slot,weight,treasure=false,cancelList=[],book=undefined){
        this.id = id;
        this.name = name;
        this.maxLevel = maxLevel;
        this.slot = slot;
        this.weight = weight;
        this.treasure = treasure;
        this.cancelList = cancelList;
        this.book = book;
    }

    setItem(item,level){
        item.setDynamicProperty(`enchant.${this.id}.valid`,true)
        item.setDynamicProperty(`enchant.${this.id}.level`,level)
        item.setDynamicProperty("hog.enchanted",true)
        return true
    }

    removeItem(item){
        const l = item.getDynamicProperty(`enchant.${this.id}.level`) > 0 ? item.getDynamicProperty(`enchant.${this.id}.level`) : 0
        item.setDynamicProperty(`enchant.${this.id}.valid`,false)
        item.setDynamicProperty(`enchant.${this.id}.level`,0)
        return l
    }

    getItem(item){
        if (item.getDynamicProperty(`enchant.${this.id}.valid`)){
            const level = item.getDynamicProperty(`enchant.${this.id}.level`)
            return {valid:true,level:level}
        }else {
            return {valid:false,level:0}
        }
    }

    toString(){
        return `{id:${this.id},name:${this.name},maxLevel:${this.maxLevel},slot:${this.slot.toString()}}`
    }
}

const regName = []
const slotRegName = []
// To register an enchantment in your enchantment file/area run this function
// id being just how to find it, mostly useless to the user
// name is the visible name it will show to players
// Max level is the maximum level the enchantment can be, without cheating
// Slot can be a string or array saying what enchantment slot it checks for e.g. sword, chest, shovel.
// Cancel list is an array of the enchantments it cannot be paired with. Being in the form of enchant ids
// For slot use the dict slots below
export function registerEnchant(id,name,maxLevel,slot,weight=1,treasure=false,cancelList=[],book=undefined){

    // This is mainly useless for you to read through

    // Setting the json
    var ench = {
        id: id,
        name: name,
        maxLevel: maxLevel,
        slot: slot,
        weight: weight,
        treasure: treasure,
        cancelList: cancelList,
        book: book
    }

    var name = JSON.stringify(ench)

    overworld.runCommand(`scriptevent hog:register_enchantment ${name}`)

    // Returns enchantment for use in the enchantment file
    // for finding if items have the enchantment on them
    return new Enchantment(id,name,maxLevel,slot,weight,treasure,cancelList,book)
}

const overworld = world.getDimension("overworld")

// Registering new slots
// E.g. the tag on an item needed to allow enchanting
export function registerSlot(id,name){
    // Setting the name
    var jsonString = `{ "id": ${id}, "name": ${name} }`

    overworld.runCommand(`scriptevent hog:register_slot ${jsonString}`)

    // Returns the valid response
    return true
}

// The names of the slots
// These are the same as the enum in script api
// Use these for any hogchantments
export const slots = new Map()
.set("feet","ArmorFeet")
.set("chest","ArmorTorso")
.set("head","ArmorHead")
.set("legs","ArmorLegs")
.set("axe","Axe")
.set("bow","Bow")
.set("carrot_stick","CarrotStick")
.set("cosmetic_head","CosmeticHead")
.set("crossbow","Crossbow")
.set("elytra","Elytra")
.set("fishing_rod","FishingRod")
.set("flintsteel","Flintsteel")
.set("hoe","Hoe")
.set("pickaxe","Pickaxe")
.set("shears","Shears")
.set("shield","Shield")
.set("shovel","Shovel")
.set("sword","Sword")
.set("trident","Spear")
.set("mace","Mace")
.set("shears","Shears");

export const slotItems = new Map()





// New updated script for parsing through new enchantments
// Honestly annoyed it took me so long to actually think to use this
// I use it so often in my biggest project i dont know how it didnt click to just use /scriptevent

system.afterEvents.scriptEventReceive.subscribe(e => {
    const player = e.sourceEntity
    if (e.id == "hog:register_slot"){
        const json = JSON.parse(e.message);
        slots.set(json.id,json.name);
    }
    if (e.id == "hog:register_item_slot"){
        const json = JSON.parse(e.message);
        slotItems.set(json.id,json.slot);
    }
})